#include "signalwidget.h"
#include "ui_signalwidget.h"

#include <QDebug>
#include <QRadioButton>
#include <memory>
#include "../items/signal.h"

SignalWidget::SignalWidget(std::weak_ptr<Item> item, QWidget *parent) :
	ItemWidget(item, parent),
	ui(new Ui::SignalWidget)
{
	ui->setupUi(this);

	if(auto workingItem = item_.lock())
	{
		std::shared_ptr<Signal> signal = std::dynamic_pointer_cast<Signal>(workingItem);
		if(signal)
		{

			ui->label->setText(workingItem->getName());

			connect(ui->radioButton_go, &QRadioButton::clicked, this, [this]()
			{
				setValue(Signal::GO);
			});
			connect(ui->radioButton_slow, &QRadioButton::clicked, this, [this]()
			{
				setValue(Signal::SLOW);
			});
			connect(ui->radioButton_stop, &QRadioButton::clicked, this, [this]()
			{
				setValue(Signal::STOP);
			});

			connect(signal.get(), &Item::valueChanged, this, &SignalWidget::moveToValue);

			if(!signal->hasSlow())
				ui->radioButton_slow->hide();
		}
	}
	else
	{
		qWarning("SignalWidget got invalid item");
		SignalWidget::disable();
	}
}

void SignalWidget::deleteItem()
{
	if(auto workingItem = item_.lock())
	{
		deleteRequest(*workingItem);
	}
}

void SignalWidget::setValue(int8_t value)
{
	if(auto workingItem = item_.lock())
		workingItem->setValue(value);
	else
		disable();
}

void SignalWidget::moveToValue(int8_t value)
{
	qDebug()<<"got value"<<value;
	ui->radioButton_go->blockSignals(true);
	ui->radioButton_slow->blockSignals(true);
	ui->radioButton_stop->blockSignals(true);

	ui->radioButton_go->setChecked(value == Signal::GO);
	ui->radioButton_slow->setChecked(value == Signal::SLOW);
	ui->radioButton_stop->setChecked(value == Signal::STOP);

	ui->radioButton_go->blockSignals(false);
	ui->radioButton_slow->blockSignals(false);
	ui->radioButton_stop->blockSignals(false);
}

void SignalWidget::disable()
{
	ui->radioButton_go->setEnabled(false);
	ui->radioButton_slow->setEnabled(false);
	ui->radioButton_stop->setEnabled(false);
}

void SignalWidget::stateChanged(int state)
{
	moveToValue(state);
}

SignalWidget::~SignalWidget()
{
	delete ui;
}
